#!/usr/bin/env python3

import configparser
import glob
import logging
import os
import traceback

from autonogame.bot import OgameBot
from autonogame.crypto import Crypto
from ogame import OGame

logging.getLogger(__name__)
logging.root.setLevel(logging.INFO)


class Config(object):
    def __init__(self) -> None:
        super().__init__()

        # create configuration folder if it does not exist
        self.app_folder = os.path.join(os.getenv("APPDATA"), "autonogame")
        self.accounts_folder = os.path.join(self.app_folder, "accounts")
        for folder in [self.app_folder, self.accounts_folder]:
            if not os.path.exists(folder):
                os.makedirs(folder)

        self.config_parser = configparser.ConfigParser()
        self.crypto = Crypto()

    def exists_account(self) -> bool:
        return len(os.listdir(self.accounts_folder)) != 0

    def get_stored_accounts(self) -> list[OgameBot]:
        accounts = []
        os.chdir(self.accounts_folder)
        if len(os.listdir(self.accounts_folder)) != 0:
            for file in glob.glob("*.ini"):
                account = {}
                self.config_parser.read(file)
                account["universe"] = self.config_parser["credentials"][
                    "universe"
                ]
                account["email"] = self.config_parser["credentials"]["email"]
                with open(
                    f"{account['universe']}_"
                    + f"{account['email']}_secret1.bin".lower(),
                    "rb",
                ) as encrypted_password_file:
                    account["password"] = self.crypto.decrypt(
                        encrypted_password_file.read()
                    )
                with open(
                    f"{account['universe']}_"
                    + f"{account['email']}_secret2.bin".lower(),
                    "rb",
                ) as encrypted_token_file:
                    account["token"] = self.crypto.decrypt(
                        encrypted_token_file.read()
                    )
                accounts.append(account)
        return accounts

    def already_logged_in(self, account_universe, account_email) -> bool:
        os.chdir(self.accounts_folder)
        for file in glob.glob("*.ini"):
            account_info = file[0:-4].lower().split("_")
            return (
                account_universe.lower() == account_info[0]
                and account_email.lower() == account_info[1]
            )

    def login_client(
        self,
        account_universe,
        account_email,
        account_password,
        account_token=None,
    ) -> OgameBot:
        empire = self.login(
            universe=account_universe,
            email=account_email,
            password=account_password,
            token=account_token,
        )

        if empire:
            # Store settings
            config = configparser.ConfigParser()
            config.add_section("credentials")
            config["credentials"]["universe"] = account_universe
            config["credentials"]["email"] = account_email
            with open(
                f"{account_universe}_{account_email}.ini".lower(), "w"
            ) as configFile:
                config.write(configFile)

            # store encrypted password
            with open(
                f"{account_universe}_{account_email}_secret1.bin".lower(), "wb"
            ) as encrypted_password_file:
                encrypted_password_file.write(
                    self.crypto.encrypt_password(account_password)
                )

            # store session token
            with open(
                f"{account_universe}_{account_email}_secret2.bin".lower(), "wb"
            ) as encrypted_token_file:
                encrypted_token_file.write(
                    self.crypto.encrypt_password(empire.token)
                )

            return OgameBot(empire)

        return False

    def login(self, universe, email, password, token=None):
        try:
            return OGame(
                universe=universe,
                username=email,
                password=password,
                token=token,
            )
        except Exception:
            logging.error(traceback.print_exc())

    def delete_account(self, universe, username):
        for file in os.listdir(self.accounts_folder):
            if file.startswith(f"{universe}_{username}".lower()):
                if os.path.exists(file):
                    os.remove(file)
                else:
                    logging.error(
                        "Error: the configuration file can not be deleted."
                    )

    def store_credentials(self, credentials):
        if not self.config_parser.has_section("credentials"):
            self.config_parser.add_section("credentials")
        self.config_parser["credentials"]["universe"] = credentials["universe"]
        self.config_parser["credentials"]["email"] = credentials["email"]
        file_prefix = f"{credentials['universe']}_{credentials['email']}"
        with open(f"{file_prefix}.ini".lower(), "w") as configFile:
            self.config_parser.write(configFile)

        # store encrypted password
        with open(
            f"{file_prefix}_secret1.bin".lower(), "wb"
        ) as encrypted_password_file:
            encrypted_password_file.write(
                self.crypto.encrypt(credentials["password"])
            )

        # store session token
        with open(
            f"{file_prefix}_secret2.bin".lower(), "wb"
        ) as encrypted_token_file:
            encrypted_token_file.write(
                self.crypto.encrypt(credentials["token"])
            )

    def save_settings(self, universe, username, settings):
        config_file = f"{universe}_{username}.ini".lower()
        self.config_parser.read(config_file)
        if not self.config_parser.has_section("settings"):
            self.config_parser.add_section("settings")
        for key, value in settings.items():
            self.config_parser["settings"][key] = str(value)
        with open(config_file, "w") as configFile:
            self.config_parser.write(configFile)

    def get_stored_settings(self, universe, username):
        config_file = f"{universe}_{username}.ini".lower()
        self.config_parser.read(config_file)

        # set default settings
        settings = {}
        # TODO use enums
        settings["build_resources"] = True
        settings["build_deposits"] = True
        settings["build_facilities"] = True

        # settings found
        if self.config_parser.has_section("settings"):
            for key, value in self.config_parser["settings"].items():
                settings[key] = value

        return settings



#!/usr/bin/env python3

import tkinter as tk
from threading import Thread
from tkinter import messagebox, ttk

from autonogame.auth import Auth
from autonogame.config import Config
from autonogame.tab_account import TabAccount
from autonogame.version import __version__


class GUI(object):
    def __init__(self):
        self.window = tk.Tk()
        self.window.title(f"autonogame v{__version__}")
        self.window.resizable(False, False)
        self.window_height = 500
        self.window_width = 630
        self.update_window_size()
        self.padx = 15
        self.pady = 15

        self.tab_parent = ttk.Notebook(self.window)
        self.initialize_add_tab()
        self.tab_parent.pack(expand=1, fill="both")

        self.auth = Auth()
        self.config = Config()
        if self.config.exists_account():
            Thread(target=self.load_accounts, args=()).start()

        self.window.bind("<Return>", self.enter_pressed)

        def on_closing():
            if messagebox.askokcancel("Quit", "Do you want to quit?"):
                self.window.destroy()

        self.window.protocol("WM_DELETE_WINDOW", on_closing)
        self.window.mainloop()

    def load_accounts(self):
        self.disable_login()
        self.tab_login_label_message_var.set("Loading...")
        self.loaded_accounts = self.auth.load_accounts()
        for loaded_account in self.loaded_accounts:
            self.add_account_tab(loaded_account)
        self.enable_login()
        self.clear_login_form()

    def update_window_size(self):
        screen_width = self.window.winfo_screenwidth()
        screen_height = self.window.winfo_screenheight()
        x_cordinate = int((screen_width / 2) - (self.window_width / 2))
        y_cordinate = int((screen_height / 2) - (self.window_height / 2))
        self.window.geometry(
            "{}x{}+{}+{}".format(
                self.window_width, self.window_height, x_cordinate, y_cordinate
            )
        )

    def disable_login(self):
        self.tab_login_entry_universe.config(state=tk.DISABLED)
        self.tab_login_entry_email.config(state=tk.DISABLED)
        self.tab_login_entry_password.config(state=tk.DISABLED)
        self.tab_login_checkbox_save.config(state=tk.DISABLED)
        self.tab_login_button.config(state=tk.DISABLED)

    def enable_login(self):
        self.tab_login_entry_universe.config(state=tk.NORMAL)
        self.tab_login_entry_email.config(state=tk.NORMAL)
        self.tab_login_entry_password.config(state=tk.NORMAL)
        self.tab_login_checkbox_save.config(state=tk.NORMAL)
        self.tab_login_button.config(state=tk.NORMAL)

    def clear_login_form(self):
        self.tab_login_entry_universe.delete(0, tk.END)
        self.tab_login_entry_email.delete(0, tk.END)
        self.tab_login_entry_password.delete(0, tk.END)
        self.tab_login_label_message_var.set("")

    def enter_pressed(self, event):
        self.login_button()

    def login_button(self):
        self.tab_login_label_message_var.set("")

        account_universe = self.tab_login_entry_universe.get()
        account_email = self.tab_login_entry_email.get()
        account_password = self.tab_login_entry_password.get()

        if not account_universe or not account_email or not account_password:
            self.tab_login_label_message_var.set("All fields are required")
            return

        if self.config.already_logged_in(account_universe, account_email):
            self.tab_login_label_message_var.set("Account already added")

        account_bot = self.auth.login_client(
            account_universe, account_email, account_password
        )
        if account_bot:
            self.add_account_tab(account_bot)
        else:
            self.tab_login_label_message_var.set("Login failed")

    def add_account_tab(self, account_bot):
        new_tab = TabAccount(account_bot)
        self.tab_parent.add(
            new_tab,
            text=account_bot.empire.universe
            + "|"
            + account_bot.empire.username,
        )
        self.tabs_count += 1
        self.update_window_size()
        self.tab_parent.select(self.tabs_count - 1)
        self.clear_login_form()

    def show_logs(self):
        pass

    def initialize_add_tab(self):
        self.tab_add = ttk.Frame(self.tab_parent)

        self.tab_login_labelframe = tk.LabelFrame(
            self.tab_add, text="Add new account"
        )
        self.tab_login_labelframe.place(relx=0.5, rely=0.5, anchor=tk.CENTER)

        self.tab_login_label_universe = tk.Label(
            self.tab_login_labelframe, text="Universe:"
        )
        self.tab_login_label_universe.grid(
            row=0, column=0, padx=self.padx, pady=self.pady
        )
        self.tab_login_entry_universe = tk.Entry(self.tab_login_labelframe)
        self.tab_login_entry_universe.grid(
            row=0, column=1, padx=self.padx, pady=self.pady
        )

        self.tab_login_label_email = tk.Label(
            self.tab_login_labelframe, text="Email:"
        )
        self.tab_login_label_email.grid(
            row=1, column=0, padx=self.padx, pady=self.pady
        )
        self.tab_login_entry_email = tk.Entry(self.tab_login_labelframe)
        self.tab_login_entry_email.grid(
            row=1, column=1, padx=self.padx, pady=self.pady
        )

        self.tab_login_label_password = tk.Label(
            self.tab_login_labelframe, text="Password:"
        )
        self.tab_login_label_password.grid(
            row=2, column=0, padx=self.padx, pady=self.pady
        )
        self.tab_login_entry_password = tk.Entry(
            self.tab_login_labelframe, show="*"
        )
        self.tab_login_entry_password.grid(
            row=2, column=1, padx=self.padx, pady=self.pady
        )

        self.tab_login_checkbox_save_var = tk.IntVar(value=1)
        self.tab_login_checkbox_save = tk.Checkbutton(
            self.tab_login_labelframe,
            text="Remember account",
            variable=self.tab_login_checkbox_save_var,
        )
        self.tab_login_checkbox_save.grid(row=3, column=0, columnspan=2)

        self.tab_login_button = tk.Button(
            self.tab_login_labelframe,
            text="Login",
            command=self.login_button,
            width=25,
        )
        self.tab_login_button.grid(
            row=4, column=0, columnspan=2, padx=self.padx, pady=self.pady
        )

        self.tab_login_label_message_var = tk.StringVar()
        self.tab_login_label_message = tk.Label(
            self.tab_login_labelframe,
            textvariable=self.tab_login_label_message_var,
        )
        self.tab_login_label_message.grid(
            row=5, column=0, padx=self.padx, pady=self.pady
        )

        self.tab_parent.add(self.tab_add, text="+")
        self.tabs_count = 1


def main():
    GUI()


if __name__ == "__main__":
    main()



#!/usr/bin/env python3

import random
import string

from cryptography.hazmat.primitives.ciphers.aead import AESGCM


class Crypto(object):
    def __init__(self) -> None:
        super().__init__()

        self.secret_key = b"ZdBGoq4JgWFzs0HxFo38Me0jxnrEIp1E"
        self.encoding_format = "utf-8"

    def generate_random_string(self, length):
        letters = (
            string.ascii_lowercase + string.ascii_uppercase + string.digits
        )
        return bytes(
            "".join(random.choice(letters) for _ in range(length)),
            self.encoding_format,
        )

    def decrypt(self, ciphertext):
        msg = AESGCM(self.secret_key).decrypt(
            ciphertext[:12], ciphertext[12:], None
        )
        return msg.decode(self.encoding_format)

    def encrypt(self, text):
        nonce = self.generate_random_string(12)
        return nonce + AESGCM(self.secret_key).encrypt(
            nonce, bytes(text, self.encoding_format), None
        )



#!/usr/bin/env python3

import logging
import traceback
from typing import Union

from autonogame.bot import OgameBot
from autonogame.config import Config
from autonogame.crypto import Crypto
from ogame import OGame

logging.getLogger(__name__)
logging.root.setLevel(logging.INFO)


class Auth(object):
    def __init__(self) -> None:
        super().__init__()

        self.config = Config()
        self.crypto = Crypto()

    def load_accounts(self) -> list[OgameBot]:
        logging.info("Loading accounts...")
        logged_in_accounts = []
        stored_accounts = self.config.get_stored_accounts()

        for account in stored_accounts:
            account_bot = self.login_client(
                account["universe"],
                account["email"],
                account["password"],
                account["token"],
            )
            if account_bot:
                logged_in_accounts.append(account_bot)
                logging.info(f"Account `{account['email']}` loaded")
        return logged_in_accounts

    def login_client(
        self,
        account_universe,
        account_email,
        account_password,
        account_token=None,
    ) -> Union[OgameBot, None]:
        empire = self.login(
            universe=account_universe,
            email=account_email,
            password=account_password,
            token=account_token,
        )

        if empire:
            logging.info(f"Account `{account_email}` logged in")
            credentials = {
                "universe": account_universe,
                "email": account_email,
                "password": account_password,
                "token": empire.token,
            }
            self.config.store_credentials(credentials)

            return OgameBot(empire)

        return None

    def login(self, universe, email, password, token=None):
        try:
            return OGame(
                universe=universe,
                username=email,
                password=password,
                token=token,
            )
        except Exception:
            logging.error(traceback.print_exc())

    def logout(self, account_bot: OgameBot):
        self.config.delete_account(
            universe=account_bot.empire.universe,
            username=account_bot.empire.username,
        )
        logging.info(
            f"Deleted credentials for account `{account_bot.empire.username}`"
        )



#!/usr/bin/env python3

import logging
import time
import traceback

from ogame import OGame
from ogame.constants import buildings, research

logging.getLogger(__name__)
logging.root.setLevel(logging.INFO)


class OgameBot(object):

    stop_command = False
    logout_command = False
    running = False
    # TODO add it to settings
    refresh_time = 10  # seconds

    # deposit capacity in k starting from level 0
    DEPOSIT_CAPACITY = [
        10,
        20,
        40,
        75,
        140,
        255,
        470,
        865,
        1590,
        2920,
        5355,
        9820,
        18005,
        33005,
        60510,
        110925,
    ]

    # Initial build steps based on
    # https://ogame.fandom.com/wiki/Quick_Start_Guide
    EARLY_STAGE_STEPS = [
        # (building, level)
        (buildings.solar_plant, 1),
        (buildings.metal_mine, 1),
        (buildings.metal_mine, 2),
        (buildings.solar_plant, 2),
        (buildings.metal_mine, 3),
        (buildings.metal_mine, 4),
        (buildings.solar_plant, 3),
        (buildings.crystal_mine, 1),
        (buildings.solar_plant, 4),
        (buildings.metal_mine, 5),
        (buildings.crystal_mine, 2),
        (buildings.crystal_mine, 3),
        (buildings.solar_plant, 5),
        (buildings.deuterium_mine, 1),
        (buildings.crystal_mine, 4),
        (buildings.solar_plant, 6),
        (buildings.metal_mine, 6),
        (buildings.metal_mine, 7),
        (buildings.solar_plant, 7),
        (buildings.crystal_mine, 5),
        (buildings.deuterium_mine, 2),
        (buildings.solar_plant, 8),
        (buildings.deuterium_mine, 3),
        (buildings.deuterium_mine, 4),
        (buildings.solar_plant, 9),
        (buildings.deuterium_mine, 5),
        (buildings.robotics_factory, 1),
        (buildings.robotics_factory, 2),
        (buildings.research_laboratory, 1),
        (buildings.shipyard, 1),
        (buildings.crystal_mine, 6),
        (buildings.shipyard, 2),
        (buildings.solar_plant, 10),
        (buildings.deuterium_mine, 6),
        (buildings.metal_mine, 8),
        (research.energy, 1),
        (research.combustion_drive, 1),
        (buildings.solar_plant, 11),
        (buildings.crystal_mine, 7),
        (buildings.metal_mine, 9),
    ]

    def __init__(self, empire: OGame):
        self.empire = empire
        self.planet_ids = self.empire.planet_ids()

    def attach_observer(self, notify):
        self.notify = notify

    def build_deposits(self, planet_id):
        if self.stop_command:
            return

        self.notify("build_deposits...")

        self.notify(
            f"Deposits: {self.metal_storage.level} "
            + f" {self.crystal_storage.level} "
            + f" {self.deuterium_storage.level}"
        )

        # upgrade deposit if it is at 90% of its capacity or above
        if (self.curr_planet_resources.metal * 100) / (
            self.DEPOSIT_CAPACITY[self.metal_storage.level] * 1000
        ) > 90:
            if self.metal_storage.is_possible:
                self.notify("Building metal_storage")
                self.empire.build(what=buildings.metal_storage, id=planet_id)

        if (self.curr_planet_resources.crystal * 100) / (
            self.DEPOSIT_CAPACITY[self.crystal_storage.level] * 1000
        ) > 90:
            if self.crystal_storage.is_possible:
                self.notify("Building crystal_storage")
                self.empire.build(what=buildings.crystal_storage, id=planet_id)

        if (self.curr_planet_resources.deuterium * 100) / (
            self.DEPOSIT_CAPACITY[self.deuterium_storage.level] * 1000
        ) > 90:
            if self.deuterium_storage.is_possible:
                self.notify("Building deuterium_storage")
                self.empire.build(
                    what=buildings.deuterium_storage, id=planet_id
                )

    def build_mines(self, planet_id):
        if self.stop_command:
            return

        self.notify("build_mines...")

        self.notify(
            f"Mines: {self.metal_mine.level} "
            + f"{self.crystal_mine.level} "
            + f"{self.deuterium_mine.level} {self.solar_plant.level}"
        )

        if self.curr_planet_resources.energy <= 0:
            self.notify("Energy is lower or equal than 0!")
            if self.solar_plant.is_possible:
                self.notify("Building solar_plant")
                self.empire.build(what=buildings.solar_plant, id=planet_id)
            return

        # Strategy
        # 1) Check: level of metal mine must be 2 levels higher
        #           than crystal mine.
        # 2) Check: for every 2 levels of crystal mines 1 level of
        #           deuterium mine must be built.
        # 3) Upgrade metal mine
        if self.crystal_mine.level < self.metal_mine.level - 2:
            if self.crystal_mine.is_possible:
                self.notify("Building crystal_mine")
                self.empire.build(what=buildings.crystal_mine, id=planet_id)
            return
        if self.deuterium_mine.level < self.crystal_mine.level // 2:
            if self.deuterium_mine.is_possible:
                self.notify("Building deuterium_mine")
                self.empire.build(what=buildings.deuterium_mine, id=planet_id)
            return
        if self.metal_mine.is_possible:
            self.notify("Building metal_mine")
            self.empire.build(what=buildings.metal_mine, id=planet_id)
            return

    def start(self):
        self.stop_command = False
        if self.running:
            self.notify("The bot is already running")
            return
        self.running = True
        self.notify("Started")

        while not self.stop_command:
            for planet_id in self.planet_ids:
                if self.stop_command:
                    break
                try:
                    self.update_current_planet_info(planet_id)

                    # Early stage planet strategy
                    early_stage_planet = False
                    for step_tuple, level in self.EARLY_STAGE_STEPS:

                        if step_tuple[2] == "supplies":
                            type = "Building"
                            check_level = self.curr_planet_supply
                            name = buildings.building_name(step_tuple)
                        if step_tuple[2] == "facilities":
                            type = "Building"
                            check_level = self.curr_planet_facilities
                            name = buildings.building_name(step_tuple)
                        if step_tuple[2] == "research":
                            type = "Researching"
                            check_level = self.curr_planet_research
                            name = research.research_name(step_tuple)

                        if check_level:
                            if (
                                getattr(
                                    check_level,
                                    name,
                                ).level
                                < level
                            ):
                                logging.info(
                                    f"{self.current_planet_name} "
                                    + "is in EARLY_STAGE mode"
                                )
                                early_stage_planet = True
                                if getattr(
                                    check_level,
                                    name,
                                ).is_possible:
                                    self.notify(
                                        type
                                        + " "
                                        + name
                                        + " lv. "
                                        + str(level)
                                        + " on planet "
                                        + self.current_planet_name
                                    )
                                    self.empire.build(
                                        what=step_tuple, id=planet_id
                                    )
                                # can do research and build at the same time
                                # go to next step if research
                                if type != "Research":
                                    break

                    if not early_stage_planet:
                        # Not early stage planet strategy
                        logging.info(
                            f"{self.current_planet_name} "
                            + "is not in EARLY_STAGE mode"
                        )
                        self.build_deposits(planet_id)
                        self.build_mines(planet_id)

                except Exception:
                    logging.error(traceback.print_exc())
                    self.refresh_session()
                finally:
                    self.notify(f"Updating in {self.refresh_time}s...")
                    time.sleep(self.refresh_time)
        self.running = False
        self.stop_command = False
        self.notify("Stopped")

    def refresh_session(self):
        self.notify("Refreshing session...")
        self.empire.relogin()
        self.planet_ids = self.empire.planet_ids()

    def stop(self):
        self.stop_command = True

    def logout(self):
        self.logout_command = True

    def update_current_planet_info(self, planet_id):
        # get planet name
        self.current_planet_name = self.empire.name_by_planet_id(planet_id)
        # get resources, supplies, facilities and research
        self.curr_planet_resources = self.empire.resources(planet_id)
        self.curr_planet_supply = self.empire.supply(planet_id)
        self.curr_planet_facilities = self.empire.facilities(planet_id)
        self.curr_planet_research = self.empire.research(planet_id)
        # update resources
        self.metal_mine = self.curr_planet_supply.metal_mine
        self.crystal_mine = self.curr_planet_supply.crystal_mine
        self.deuterium_mine = self.curr_planet_supply.deuterium_mine
        self.solar_plant = self.curr_planet_supply.solar_plant
        # update deposits
        self.metal_storage = self.curr_planet_supply.metal_storage
        self.crystal_storage = self.curr_planet_supply.crystal_storage
        self.deuterium_storage = self.curr_planet_supply.deuterium_storage



from distutils.util import convert_path
from setuptools import find_packages, setup

module_name = "autonogame"
main_ns = {}
ver_path = convert_path(f"src/{module_name}/version.py")
with open(ver_path) as ver_file:
    exec(ver_file.read(), main_ns)

setup(
    name=module_name,
    version=main_ns["__version__"],
    packages=find_packages(where="src"),
    package_dir={"": "src"},
    license="MIT",
    description="Ogame bot",
    keywords=["ogame", "bot", "hack", "script"],
    long_description_content_type="text/markdown",
    long_description=open("README.md", encoding="utf8").read(),
    install_requires=[
        "cryptography==37.0.2",
        "ogame==8.4.0.22",
    ],
    url="https://github.com/tassoneroberto/autonogame",
    author="Roberto Tassone",
    author_email="roberto.tassone@proton.me",
    classifiers=[
        "Development Status :: 2 - Pre-Alpha",
        "Intended Audience :: Developers",
        "Natural Language :: English",
        "Operating System :: Microsoft :: Windows",
        "Topic :: Software Development :: Build Tools",
        "Topic :: Software Development :: Libraries",
        "License :: OSI Approved :: MIT License",
        "Programming Language :: Python :: 3.10",
    ],
    entry_points={
        "console_scripts": [
            "autonogame = autonogame.gui:main",
        ],
    },
)



#!/usr/bin/env python3

import logging
import threading
import tkinter as tk
from tkinter import ttk

from autonogame.auth import Auth
from autonogame.bot import OgameBot
from autonogame.config import Config

logging.getLogger(__name__)
logging.root.setLevel(logging.INFO)


class TabAccount(ttk.Frame):
    def __init__(self, bot: OgameBot):
        super().__init__()
        self.bot = bot
        self.auth = Auth()
        self.config = Config()
        self.settings = self.config.get_stored_settings(
            self.bot.empire.universe, self.bot.empire.username
        )
        self.bot.attach_observer(self.notify)

        # User info frame
        self.tab_account_labelframe_user = tk.LabelFrame(
            self, text="User info"
        )
        self.tab_account_labelframe_user.grid(
            row=0, column=0, padx=5, pady=5, sticky=tk.W
        )

        self.label_email = tk.Label(
            self.tab_account_labelframe_user,
            text=f"Email: {self.bot.empire.username}",
        )
        self.label_email.grid(row=0, column=0, padx=5, pady=0, sticky=tk.W)

        self.label_server_number = tk.Label(
            self.tab_account_labelframe_user,
            text=f"Server number: {self.bot.empire.server_number}",
        )
        self.label_server_number.grid(
            row=1, column=0, padx=5, pady=0, sticky=tk.W
        )

        self.label_server_id = tk.Label(
            self.tab_account_labelframe_user,
            text=f"Server ID: {self.bot.empire.server_id}",
        )
        self.label_server_id.grid(row=2, column=0, padx=5, pady=0, sticky=tk.W)

        self.label_language = tk.Label(
            self.tab_account_labelframe_user,
            text=f"Language: {self.bot.empire.language}",
        )
        self.label_language.grid(row=3, column=0, padx=5, pady=0, sticky=tk.W)

        self.label_universe = tk.Label(
            self.tab_account_labelframe_user,
            text=f"Universe: {self.bot.empire.universe}",
        )
        self.label_universe.grid(row=4, column=0, padx=5, pady=0, sticky=tk.W)

        self.button_logout = tk.Button(
            self.tab_account_labelframe_user,
            text="Logout",
            command=self.logout,
            width=25,
        )
        self.button_logout.grid(row=5, column=0, padx=5, pady=5, sticky=tk.W)

        # Bot settings frame
        self.tab_settings_labelframe_user = tk.LabelFrame(
            self, text="Settings"
        )
        self.tab_settings_labelframe_user.grid(
            row=0, column=1, padx=5, pady=5, sticky=tk.W
        )

        self.tab_settings_check_buildres_var = tk.BooleanVar(
            value=self.settings["build_resources"]
        )
        self.tab_settings_check_buildres = tk.Checkbutton(
            self.tab_settings_labelframe_user,
            text="Build resources",
            variable=self.tab_settings_check_buildres_var,
        )
        self.tab_settings_check_buildres.grid(
            row=0, column=0, padx=5, pady=5, sticky=tk.W
        )

        self.tab_settings_check_builddepot_var = tk.BooleanVar(
            value=self.settings["build_deposits"]
        )
        self.tab_settings_check_builddepot = tk.Checkbutton(
            self.tab_settings_labelframe_user,
            text="Build deposits",
            variable=self.tab_settings_check_builddepot_var,
        )
        self.tab_settings_check_builddepot.grid(
            row=1, column=0, padx=5, pady=5, sticky=tk.W
        )

        self.tab_settings_check_buildfac_var = tk.BooleanVar(
            value=self.settings["build_facilities"]
        )
        self.tab_settings_check_buildfac = tk.Checkbutton(
            self.tab_settings_labelframe_user,
            text="Build facilities",
            variable=self.tab_settings_check_buildfac_var,
        )
        self.tab_settings_check_buildfac.grid(
            row=2, column=0, padx=5, pady=5, sticky=tk.W
        )

        self.button_save_settings = tk.Button(
            self.tab_settings_labelframe_user,
            text="Save",
            command=self.save_settings,
            width=25,
        )
        self.button_save_settings.grid(
            row=3, column=0, padx=5, pady=5, sticky=tk.W
        )

        # Bot commands frame
        self.tab_account_labelframe_commands = tk.LabelFrame(self, text="Bot")
        self.tab_account_labelframe_commands.grid(
            row=0, column=2, padx=5, pady=5, sticky=tk.W
        )

        self.label_botstatus = tk.Label(
            self.tab_account_labelframe_commands, text="Status: idle"
        )
        self.label_botstatus.grid(row=0, column=0, padx=5, pady=5, sticky=tk.W)

        self.button_start = tk.Button(
            self.tab_account_labelframe_commands,
            text="Start",
            command=self.start_bot,
            width=20,
        )
        self.button_start.grid(row=1, column=0, padx=5, pady=5, sticky=tk.W)

        self.button_stop = tk.Button(
            self.tab_account_labelframe_commands,
            text="Stop",
            command=self.stop_bot,
            width=20,
            state=tk.DISABLED,
        )
        self.button_stop.grid(row=2, column=0, padx=5, pady=5, sticky=tk.W)

        # Logs
        self.text_logs = tk.Text(self, height="18", width="76")
        self.text_logs.grid(row=2, column=0, padx=5, pady=5, columnspan=3)

    def notify(self, message: str):
        if self.bot.logout_command:
            return
        self.text_logs.insert("1.0", message + "\n")
        logging.info(f"Bot: {message}")
        self.update_bot_status()

    def update_bot_status(self):
        bot_status_text = "running"
        if not self.bot.running:
            bot_status_text = "stopped"
        if self.bot.running and self.bot.stop_command:
            bot_status_text = "stopping"
        self.label_botstatus.config(text=f"Status: {bot_status_text}")

    def start_bot(self):
        self.disable_buttons_start_pressed()
        self.notify("Starting...")
        x = threading.Thread(target=self.bot.start)
        x.start()

    def stop_bot(self):
        if not self.bot.running:
            self.notify("The bot is already stopped")
        elif self.bot.stop_command:
            self.notify("The bot is already stopping...")
        else:
            self.enable_buttons_stop_pressed()
            self.notify("Stopping...")
            self.bot.stop()

    def disable_buttons_start_pressed(self):
        # enable stop button
        self.button_stop.config(state=tk.NORMAL)
        # disable all
        self.button_logout.config(state=tk.DISABLED)
        self.tab_settings_check_buildres.config(state=tk.DISABLED)
        self.tab_settings_check_buildfac.config(state=tk.DISABLED)
        self.tab_settings_check_builddepot.config(state=tk.DISABLED)
        self.button_save_settings.config(state=tk.DISABLED)
        self.button_start.config(state=tk.DISABLED)

    def enable_buttons_stop_pressed(self):
        # disable stop button
        self.button_stop.config(state=tk.DISABLED)
        # enable all
        self.button_logout.config(state=tk.NORMAL)
        self.tab_settings_check_buildres.config(state=tk.NORMAL)
        self.tab_settings_check_buildfac.config(state=tk.NORMAL)
        self.tab_settings_check_builddepot.config(state=tk.NORMAL)
        self.button_save_settings.config(state=tk.NORMAL)
        self.button_start.config(state=tk.NORMAL)

    def logout(self):
        # stop notify
        self.bot.logout()
        # stop bot
        self.stop_bot()
        # delete credentials
        self.auth.logout(self.bot)
        # remove tab from gui
        self.pack_forget()
        self.destroy()
        logging.info("Logged out")

    def save_settings(self):
        settings = {
            "build_resources": self.tab_settings_check_buildres_var.get(),
            "build_deposits": self.tab_settings_check_builddepot_var.get(),
            "build_facilities": self.tab_settings_check_buildfac_var.get(),
        }
        self.config.save_settings(
            universe=self.bot.empire.universe,
            username=self.bot.empire.username,
            settings=settings,
        )
        self.notify("Settings saved")
